@@ -1,6 +1,6 @@
 /* Python pretty-printing
 
-   Copyright (C) 2008-2019 Free Software Foundation, Inc.
+   Copyright (C) 2008-2022 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -29,7 +29,7 @@
 
 /* Return type of print_string_repr.  */
 
-enum string_repr_result
+enum gdbpy_string_repr_result
   {
     /* The string method returned None.  */
     string_repr_none,
@@ -39,6 +39,10 @@ enum string_repr_result
     string_repr_ok
   };
 
+/* If non-null, points to options that are in effect while
+   printing.  */
+const struct value_print_options *gdbpy_current_print_options;
+
 /* Helper function for find_pretty_printer which iterates over a list,
    calls each function and inspects output.  This will return a
    printer object if one recognizes VALUE.  If no printer is found, it
@@ -273,7 +277,7 @@ print_stack_unless_memory_error (struct ui_file *stream)
 /* Helper for gdbpy_apply_val_pretty_printer which calls to_string and
    formats the result.  */
 
-static enum string_repr_result
+static enum gdbpy_string_repr_result
 print_string_repr (PyObject *printer, const char *hint,
 		   struct ui_file *stream, int recurse,
 		   const struct value_print_options *options,
@@ -281,7 +285,7 @@ print_string_repr (PyObject *printer, const char *hint,
 		   struct gdbarch *gdbarch)
 {
   struct value *replacement = NULL;
-  enum string_repr_result result = string_repr_ok;
+  enum gdbpy_string_repr_result result = string_repr_ok;
 
   gdbpy_ref<> py_str = pretty_print_one_value (printer, &replacement);
   if (py_str != NULL)
@@ -318,10 +322,10 @@ print_string_repr (PyObject *printer, const char *hint,
 	      type = builtin_type (gdbarch)->builtin_char;
 
 	      if (hint && !strcmp (hint, "string"))
-		LA_PRINT_STRING (stream, type, (gdb_byte *) output,
-				 length, NULL, 0, options);
+		language->printstr (stream, type, (gdb_byte *) output,
+				    length, NULL, 0, options);
 	      else
-		fputs_filtered (output, stream);
+		gdb_puts (output, stream);
 	    }
 	  else
 	    {
@@ -425,27 +429,34 @@ print_children (PyObject *printer, const char *hint,
 	  /* The user won't necessarily get a stack trace here, so provide
 	     more context.  */
 	  if (gdbpy_print_python_errors_p ())
-	    fprintf_unfiltered (gdb_stderr,
-				_("Bad result from children iterator.\n"));
+	    gdb_printf (gdb_stderr,
+			_("Bad result from children iterator.\n"));
 	  gdbpy_print_stack ();
 	  continue;
 	}
 
-      /* Print initial "{".  For other elements, there are three
-	 cases:
+      /* Print initial "=" to separate print_string_repr output and
+	 children.  For other elements, there are three cases:
 	 1. Maps.  Print a "," after each value element.
 	 2. Arrays.  Always print a ",".
 	 3. Other.  Always print a ",".  */
       if (i == 0)
 	{
-         if (is_py_none)
-           fputs_filtered ("{", stream);
-         else
-           fputs_filtered (" = {", stream);
-       }
-
+	  if (!is_py_none)
+	    gdb_puts (" = ", stream);
+	}
       else if (! is_map || i % 2 == 0)
-	fputs_filtered (pretty ? "," : ", ", stream);
+	gdb_puts (pretty ? "," : ", ", stream);
+
+      /* Skip printing children if max_depth has been reached.  This check
+	 is performed after print_string_repr and the "=" separator so that
+	 these steps are not skipped if the variable is located within the
+	 permitted depth.  */
+      if (val_print_check_max_depth (stream, recurse, options, language))
+	return;
+      else if (i == 0)
+	/* Print initial "{" to bookend children.  */
+	gdb_puts ("{", stream);
 
       /* In summary mode, we just want to print "= {...}" if there is
 	 a value.  */
@@ -463,26 +474,26 @@ print_children (PyObject *printer, const char *hint,
 	{
 	  if (pretty)
 	    {
-	      fputs_filtered ("\n", stream);
-	      print_spaces_filtered (2 + 2 * recurse, stream);
+	      gdb_puts ("\n", stream);
+	      print_spaces (2 + 2 * recurse, stream);
 	    }
 	  else
-	    wrap_here (n_spaces (2 + 2 *recurse));
+	    stream->wrap_here (2 + 2 *recurse);
 	}
 
       if (is_map && i % 2 == 0)
-	fputs_filtered ("[", stream);
+	gdb_puts ("[", stream);
       else if (is_array)
 	{
 	  /* We print the index, not whatever the child method
 	     returned as the name.  */
 	  if (options->print_array_indexes)
-	    fprintf_filtered (stream, "[%d] = ", i);
+	    gdb_printf (stream, "[%d] = ", i);
 	}
       else if (! is_map)
 	{
-	  fputs_filtered (name, stream);
-	  fputs_filtered (" = ", stream);
+	  gdb_puts (name, stream);
+	  gdb_puts (" = ", stream);
 	}
 
       if (gdbpy_is_lazy_string (py_v))
@@ -507,7 +518,7 @@ print_children (PyObject *printer, const char *hint,
 	  if (!output)
 	    gdbpy_print_stack ();
 	  else
-	    fputs_filtered (output.get (), stream);
+	    gdb_puts (output.get (), stream);
 	}
       else
 	{
@@ -533,7 +544,7 @@ print_children (PyObject *printer, const char *hint,
 	}
 
       if (is_map && i % 2 == 0)
-	fputs_filtered ("] = ", stream);
+	gdb_puts ("] = ", stream);
     }
 
   if (i)
@@ -542,38 +553,36 @@ print_children (PyObject *printer, const char *hint,
 	{
 	  if (pretty)
 	    {
-	      fputs_filtered ("\n", stream);
-	      print_spaces_filtered (2 + 2 * recurse, stream);
+	      gdb_puts ("\n", stream);
+	      print_spaces (2 + 2 * recurse, stream);
 	    }
-	  fputs_filtered ("...", stream);
+	  gdb_puts ("...", stream);
 	}
       if (pretty)
 	{
-	  fputs_filtered ("\n", stream);
-	  print_spaces_filtered (2 * recurse, stream);
+	  gdb_puts ("\n", stream);
+	  print_spaces (2 * recurse, stream);
 	}
-      fputs_filtered ("}", stream);
+      gdb_puts ("}", stream);
     }
 }
 
 enum ext_lang_rc
 gdbpy_apply_val_pretty_printer (const struct extension_language_defn *extlang,
-				struct type *type,
-				LONGEST embedded_offset, CORE_ADDR address,
+				struct value *value,
 				struct ui_file *stream, int recurse,
-				struct value *val,
 				const struct value_print_options *options,
 				const struct language_defn *language)
 {
-  struct gdbarch *gdbarch = get_type_arch (type);
-  struct value *value;
-  enum string_repr_result print_result;
+  struct type *type = value_type (value);
+  struct gdbarch *gdbarch = type->arch ();
+  enum gdbpy_string_repr_result print_result;
 
-  if (value_lazy (val))
-    value_fetch_lazy (val);
+  if (value_lazy (value))
+    value_fetch_lazy (value);
 
   /* No pretty-printer support for unavailable values.  */
-  if (!value_bytes_available (val, embedded_offset, TYPE_LENGTH (type)))
+  if (!value_bytes_available (value, 0, type->length ()))
     return EXT_LANG_RC_NOP;
 
   if (!gdb_python_initialized)
@@ -581,10 +590,7 @@ gdbpy_apply_val_pretty_printer (const struct extension_language_defn *extlang,
 
   gdbpy_enter enter_py (gdbarch, language);
 
-  /* Instantiate the printer.  */
-  value = value_from_component (val, type, embedded_offset);
-
-  gdbpy_ref<> val_obj (value_to_value_object (value));
+  gdbpy_ref<> val_obj (value_to_value_object_no_release (value));
   if (val_obj == NULL)
     {
       print_stack_unless_memory_error (stream);
@@ -602,8 +608,8 @@ gdbpy_apply_val_pretty_printer (const struct extension_language_defn *extlang,
   if (printer == Py_None)
     return EXT_LANG_RC_NOP;
 
-  if (val_print_check_max_depth (stream, recurse, options, language))
-    return EXT_LANG_RC_OK;
+  scoped_restore set_options = make_scoped_restore (&gdbpy_current_print_options,
+						    options);
 
   /* If we are printing a map, we want some special formatting.  */
   gdb::unique_xmalloc_ptr<char> hint (gdbpy_get_display_hint (printer.get ()));
@@ -633,8 +639,12 @@ gdbpy_apply_val_pretty_printer (const struct extension_language_defn *extlang,
 gdbpy_ref<>
 apply_varobj_pretty_printer (PyObject *printer_obj,
 			     struct value **replacement,
-			     struct ui_file *stream)
+			     struct ui_file *stream,
+			     const value_print_options *opts)
 {
+  scoped_restore set_options = make_scoped_restore (&gdbpy_current_print_options,
+						    opts);
+
   *replacement = NULL;
   gdbpy_ref<> py_str = pretty_print_one_value (printer_obj, replacement);
 
@@ -689,3 +699,92 @@ gdbpy_default_visualizer (PyObject *self, PyObject *args)
 
   return find_pretty_printer (val_obj).release ();
 }
+
+/* Helper function to set a boolean in a dictionary.  */
+static int
+set_boolean (PyObject *dict, const char *name, bool val)
+{
+  gdbpy_ref<> val_obj (PyBool_FromLong (val));
+  if (val_obj == nullptr)
+    return -1;
+  return PyDict_SetItemString (dict, name, val_obj.get ());
+}
+
+/* Helper function to set an integer in a dictionary.  */
+static int
+set_unsigned (PyObject *dict, const char *name, unsigned int val)
+{
+  gdbpy_ref<> val_obj = gdb_py_object_from_ulongest (val);
+  if (val_obj == nullptr)
+    return -1;
+  return PyDict_SetItemString (dict, name, val_obj.get ());
+}
+
+/* Implement gdb.print_options.  */
+PyObject *
+gdbpy_print_options (PyObject *unused1, PyObject *unused2)
+{
+  gdbpy_ref<> result (PyDict_New ());
+  if (result == nullptr)
+    return nullptr;
+
+  value_print_options opts;
+  gdbpy_get_print_options (&opts);
+
+  if (set_boolean (result.get (), "raw",
+		   opts.raw) < 0
+      || set_boolean (result.get (), "pretty_arrays",
+		      opts.prettyformat_arrays) < 0
+      || set_boolean (result.get (), "pretty_structs",
+		      opts.prettyformat_structs) < 0
+      || set_boolean (result.get (), "array_indexes",
+		      opts.print_array_indexes) < 0
+      || set_boolean (result.get (), "symbols",
+		      opts.symbol_print) < 0
+      || set_boolean (result.get (), "unions",
+		      opts.unionprint) < 0
+      || set_boolean (result.get (), "address",
+		      opts.addressprint) < 0
+      || set_boolean (result.get (), "deref_refs",
+		      opts.deref_ref) < 0
+      || set_boolean (result.get (), "actual_objects",
+		      opts.objectprint) < 0
+      || set_boolean (result.get (), "static_members",
+		      opts.static_field_print) < 0
+      || set_boolean (result.get (), "deref_refs",
+		      opts.deref_ref) < 0
+      || set_boolean (result.get (), "nibbles",
+		      opts.nibblesprint) < 0
+      || set_boolean (result.get (), "summary",
+		      opts.summary) < 0
+      || set_unsigned (result.get (), "max_elements",
+		       opts.print_max) < 0
+      || set_unsigned (result.get (), "max_depth",
+		       opts.max_depth) < 0
+      || set_unsigned (result.get (), "repeat_threshold",
+		       opts.repeat_count_threshold) < 0)
+    return nullptr;
+
+  if (opts.format != 0)
+    {
+      char str[2] = { (char) opts.format, 0 };
+      gdbpy_ref<> fmtstr = host_string_to_python_string (str);
+      if (fmtstr == nullptr)
+	return nullptr;
+      if (PyDict_SetItemString (result.get (), "format", fmtstr.get ()) < 0)
+	return nullptr;
+    }
+
+  return result.release ();
+}
+
+/* Helper function that either finds the prevailing print options, or
+   calls get_user_print_options.  */
+void
+gdbpy_get_print_options (value_print_options *opts)
+{
+  if (gdbpy_current_print_options != nullptr)
+    *opts = *gdbpy_current_print_options;
+  else
+    get_user_print_options (opts);
+}
