@@ -1,6 +1,6 @@
 /* Code dealing with register stack frames, for GDB, the GNU debugger.
 
-   Copyright (C) 1986-2017 Free Software Foundation, Inc.
+   Copyright (C) 1986-2022 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -42,7 +42,7 @@ sentinel_frame_cache (struct regcache *regcache)
 /* Here the register value is taken direct from the register cache.  */
 
 static struct value *
-sentinel_frame_prev_register (struct frame_info *this_frame,
+sentinel_frame_prev_register (frame_info_ptr this_frame,
 			      void **this_prologue_cache,
 			      int regnum)
 {
@@ -50,25 +50,25 @@ sentinel_frame_prev_register (struct frame_info *this_frame,
     = (struct frame_unwind_cache *) *this_prologue_cache;
   struct value *value;
 
-  value = regcache_cooked_read_value (cache->regcache, regnum);
+  value = cache->regcache->cooked_read_value (regnum);
   VALUE_NEXT_FRAME_ID (value) = sentinel_frame_id;
 
   return value;
 }
 
 static void
-sentinel_frame_this_id (struct frame_info *this_frame,
+sentinel_frame_this_id (frame_info_ptr this_frame,
 			void **this_prologue_cache,
 			struct frame_id *this_id)
 {
   /* The sentinel frame is used as a starting point for creating the
      previous (inner most) frame.  That frame's THIS_ID method will be
      called to determine the inner most frame's ID.  Not this one.  */
-  internal_error (__FILE__, __LINE__, _("sentinel_frame_this_id called"));
+  internal_error (_("sentinel_frame_this_id called"));
 }
 
 static struct gdbarch *
-sentinel_frame_prev_arch (struct frame_info *this_frame,
+sentinel_frame_prev_arch (frame_info_ptr this_frame,
 			  void **this_prologue_cache)
 {
   struct frame_unwind_cache *cache
@@ -79,6 +79,7 @@ sentinel_frame_prev_arch (struct frame_info *this_frame,
 
 const struct frame_unwind sentinel_frame_unwind =
 {
+  "sentinel",
   SENTINEL_FRAME,
   default_frame_unwind_stop_reason,
   sentinel_frame_this_id,
